from dataclasses import dataclass, field
from typing import Optional, List
from enum import Enum

@dataclass
class Index:
  id: str
  title: str
  signer_function: str
  signer_public_key: str
  did_owned: bool
  did_starred: bool
  roles_owner: bool
  roles_creator: bool
  owner_did: 'User'
  created_at: str
  updated_at: str
  deleted_at: Optional[str] = None
  links: List['Link'] = field(default_factory=list)

@dataclass
class User:
  id: str
  name: Optional[str] = None
  bio: Optional[str] = None
  avatar: Optional[str] = None
  created_at: Optional[str] = None
  updated_at: Optional[str] = None

@dataclass
class Link:
  id: str
  index_id: Optional[str] = None
  indexer_did: Optional[str] = None
  content: Optional[str] = None
  title: Optional[str] = None
  url: Optional[str] = None
  description: Optional[str] = None
  language: Optional[str] = None
  favicon: Optional[str] = None
  created_at: Optional[str] = None
  updated_at: Optional[str] = None
  deleted_at: Optional[str] = None
  images: List[str] = field(default_factory=list)
  favorite: bool = False
  tags: List[str] = field(default_factory=list)

class Participant(Enum):
  User = "user"
  Assistant = "assistant"

@dataclass
class Message:
  content: str
  role: Participant



import base64
import secrets
import json
import os
from dataclasses import asdict
import requests
from siwe.siwe import VersionEnum
from .config import IndexConfig
from .index_types import User, Link
from .utils import create_siwe_message, get_signature, prepare_siwe_message, random_string, siwe_message_to_json, to_iso_format
from web3 import Web3
from datetime import datetime, timedelta
from Crypto.Random import get_random_bytes
from eth_account.messages import encode_defunct
from siwe import SiweMessage
from urllib.parse import urlencode
import subprocess
from enum import Enum

class IndexClient:
    def __init__(self, domain, session=None, private_key=None, wallet=None, network="ethereum"):
      self.session = session
      self.private_key = private_key
      self.domain = domain
      self.wallet = wallet
      self.network = network
      if self.network == "mainnet":
        self.base_url = "https://index.network/api"
      else:
        self.base_url = "https://dev.index.network/api"

    def call_did(self, method, params):
      base_path = os.path.dirname(os.path.abspath(__file__))
      session_path = os.path.join(base_path, 'bin', 'session.bin')
      command = [session_path, method] + params
      result = subprocess.run(command, text=True, capture_output=True)
      return result.stdout.strip()

    def authenticate(self, session=None):
      if session:
          self.session = session
          return

      if not self.wallet or not self.domain:
        raise Exception("Private key and domain are required to authenticate")

      key_seed = get_random_bytes(32)
      key_seed_str = base64.b64encode(key_seed).decode('utf-8')
      did_key_id = self.call_did("key", [key_seed_str])

      now = datetime.utcnow()
      one_month_later = now + timedelta(days=30)

      nonce = random_string(10)
      siwe_data = {
        "domain": self.domain,
        "address": self.wallet.address,
        "statement": "Give this application access to some of your data on Ceramic",
        "uri": did_key_id,
        "version": VersionEnum.one,
        "chain_id": 1,
        "issued_at": to_iso_format(now),
        "expiration_time": to_iso_format(one_month_later),
        "nonce": nonce,
        "resources": ['ceramic://*']
      }

      siwe_message = create_siwe_message(siwe_data)
      signed_message = self.wallet.sign_message(
        encode_defunct(text=prepare_siwe_message(siwe_message))
      )
      signature = get_signature(signed_message)
      siwe_message_str = siwe_message_to_json(siwe_message)

      session = self.call_did("session", [siwe_message_str, signature, key_seed_str])
      self.session = session

    def _request(self, endpoint, method="GET", data=None):
      url = f"{self.base_url}{endpoint}"
      headers = {
        "Authorization": f"Bearer {self.session}",
        "Content-Type": "application/json"
      }
      response = requests.request(method, url, headers=headers, json=data)
      return response.json()

    def get_all_indexes(self, did):
      return self._request(f"/dids/{did}/indexes", "GET")


    def get_profile(self, did):
      return self._request(f"/dids/{did}/profile", "GET")

    def get_index(self, index_id):
      return self._request(f"/indexes/{index_id}", "GET")

    def update_profile(self, params):
      return self._request("/profile", "PATCH", json.dumps(params))

    def update_index(self, id, index):
      return self._request(f"/indexes/{id}", "PATCH", index)

    def crawl_web_page(self, url):
      return self._request("/web2/webpage/crawl", "POST", {"url": url})

    def get_lit_actions(self, cid):
      return self._request(f"/lit_actions/{cid}", "GET")

    def post_lit_action(self, action):
      return self._request("/lit_actions", "POST", action)

    def get_items(self, index_id, query_params):
      if query_params:
        query_string = urlencode(query_params)
      else:
        query_string = ""
      return self._request(f"/indexes/{index_id}/items?{query_string}", "GET")

    def create_index(self, title, signer_function=None):
      body = {
        "title": title,
      }

      if signer_function:
        body["signer_function"] = signer_function

      return self._request("/indexes", "POST", body)

    def add_item_to_index(self, index_id, item_id):
      return self._request(f"/indexes/{index_id}/items/{item_id}", "POST")

    def delete_item_from_index(self, index_id, item_id):
      return self._request(f"/indexes/{index_id}/items/{item_id}", "DELETE")

    def get_ens_name_details(self, ens_name):
      return self._request(f"/ens/{ens_name}", "GET")

    def create_item(self, index_id, item):
      return self._request(f"/indexes/{index_id}/items", "POST", item)

    def delete_item(self, index_id, item_id):
      return self._request(f"/indexes/{index_id}/items/{item_id}", "DELETE")

    def star_index(self, did, index_id):
      return self._request(f"/dids/{did}/indexes/{index_id}/star", "PUT")

    def unstar_index(self, did, index_id):
      return self._request(f"/dids/{did}/indexes/{index_id}/star", "DELETE")

    def own_index(self, did, index_id):
      return self._request(f"/dids/{did}/indexes/{index_id}/own", "PUT")

    def disown_index(self, did, index_id):
      return self._request(f"/dids/{did}/indexes/{index_id}/own", "DELETE")

    def get_nft_metadata(self, network, address, token_id=None):
      endpoint = f"/nft/{network}/{address}"
      if token_id:
        endpoint += f"/{token_id}"
      return self._request(endpoint, "GET")

    def resolve_ens(self, ens_name):
      return self._request(f"/ens/{ens_name}", "GET")

    def get_node_by_id(self, model_id, node_id):
      return self._request(f"/composedb/{model_id}/{node_id}", "GET")

    def create_node(self, model_id, node_data):
      return self._request(f"/composedb/{model_id}", "POST", node_data)

    def update_node(self, model_id, node_id, node_data):
      return self._request(f"/composedb/{model_id}/{node_id}", "PATCH", node_data)



import datetime
from enum import Enum
import random
import string
import json

from pydantic import ValidationError
from siwe import SiweMessage

def random_string(length: int) -> str:
  return ''.join(random.choices(string.ascii_letters + string.digits, k=length))

def to_iso_format(date: datetime.datetime) -> str:
  return date.isoformat(timespec='milliseconds') + "Z"

def json_serial(obj):
  """JSON serializer for objects not serializable by default json code"""
  if isinstance(obj, datetime):
    return obj.isoformat()  # Convert datetime object to ISO format
  if isinstance(obj, Enum):
    return obj.value  # Convert Enum to its value
  raise TypeError ("Type %s not serializable" % type(obj))

def prepare_siwe_message(message) -> str:
  """Serialize to the EIP-4361 format for signing.

  It can then be passed to an EIP-191 signing function.

  :return: EIP-4361 formatted message, ready for EIP-191 signing.
  """
  header = f"{message.domain} wants you to sign in with your Ethereum account:"

  uri_field = f"URI: {message.uri}"

  prefix = "\n".join([header, message.address])

  version_field = f"Version: {message.version}"

  chain_field = f"Chain ID: {message.chain_id or 1}"

  nonce_field = f"Nonce: {message.nonce}"

  suffix_array = [uri_field, version_field, chain_field, nonce_field]

  issued_at_field = f"Issued At: {message.issued_at}"
  suffix_array.append(issued_at_field)

  if message.expiration_time:
    expiration_time_field = f"Expiration Time: {message.expiration_time}"
    suffix_array.append(expiration_time_field)

  if message.not_before:
    not_before_field = f"Not Before: {message.not_before}"
    suffix_array.append(not_before_field)

  if message.request_id:
    request_id_field = f"Request ID: {message.request_id}"
    suffix_array.append(request_id_field)

  if message.resources:
    resources_field = "\n".join(
      ["Resources:"] + [f"- {resource}" for resource in message.resources]
    )
    suffix_array.append(resources_field)

  suffix = "\n".join(suffix_array)

  if message.statement:
    prefix = "\n\n".join([prefix, message.statement])
  else:
    prefix += "\n"

  return "\n\n".join([prefix, suffix])

def siwe_message_to_json(siwe_message):
  if not hasattr(siwe_message, '__dict__'):
    raise ValueError("The provided siwe_message doesn't seem to be an object with attributes.")

  siwe_data = {attr: getattr(siwe_message, attr) for attr in vars(siwe_message)}

  return json.dumps(siwe_data, default=json_serial)

def create_siwe_message(data):
  original_uri = data['uri']
  original_resources = data['resources']
  data['uri'] = 'https://example.com'
  data['resources'] = ['https://example.com']

  try:
    message = SiweMessage(message=data)
    message.uri = original_uri
    message.resources = original_resources
    return message
  except ValidationError as e:
    print("Validation failed:", e)
    return None

def get_signature(signed_message):
  return "0x" + signed_message.signature.hex()



from .client import IndexClient
from .chroma import IndexChroma
from .index_types import User, Index, Link, Participant, Message

__all__ = ['IndexClient', 'User', 'Index', 'Link', 'Participant', 'Message', 'IndexChroma']



import chromadb
from chromadb import Settings
from langchain_chroma import Chroma
from langchain_openai import OpenAIEmbeddings

from .config import IndexConfig
from typing_extensions import override


class IndexChroma(Chroma):
    """Read-only Chroma Client for Index Network
    Desciption:
        This class is a read-only Chroma client for Index Network. It is used to query the decentralized semantic network provided by Index Network.

    Args:
        embedding_function (OpenAIEmbeddings): OpenAIEmbeddings object that contains the API key and the model name.

    Usage:

    * Initialize IndexChroma object

        ```python
        from langchain_openai import OpenAIEmbeddings
        from langchain_index import IndexChroma

        # Initialize OpenAIEmbeddings object

        embeddings = OpenAIEmbeddings(model="text-embedding-ada-002",
                                      openai_api_key="YOUR_API_KEY")

        vectorstore = IndexChroma(embedding_function=embeddings)
        ```

    * Query the network

        ```python
        # Query the network
        response = vectorstore.query("What is the capital of France?")
        print(response)
        ```

    * Also you can use with LangChain as follows:

        ```python
        from langchain import hub
        from langchain_openai import ChatOpenAI
        from langchain_core.output_parsers import StrOutputParser
        from langchain_core.runnables import RunnablePassthrough

        llm = ChatOpenAI(model="gpt-3.5-turbo-0125", api_key='YOUR_API_KEY')

        # Retrieve and generate using the relevant snippets of the blog.
        retriever = vectorstore.as_retriever()
        prompt = hub.pull("rlm/rag-prompt")

        def format_docs(docs):
            return "\\n\\n".join(doc.page_content for doc in docs)

        rag_chain = (
            {"context": retriever | format_docs, "question": RunnablePassthrough()}
            | prompt
            | llm
            | StrOutputParser()
        )

        rag_chain.invoke("What is $STYLE Protocol?")


    """

    def __init__(self, embedding_function: OpenAIEmbeddings):
        chroma_path = "https://dev.index.network:8000/chroma"
        if self.network == "mainnet":
            chroma_path = "https://index.network:8000/chroma"

        # Initialize chroma client
        self.client = chromadb.HttpClient(
                            settings=Settings(
                                chroma_server_api_default_path=chroma_path,
                                anonymized_telemetry=False,
                                allow_reset=True
                        ))

        super().__init__(client=self.client,
                         embedding_function=embedding_function,
                         collection_name=IndexConfig.CHROMA_COLLECTION)


    @override
    def add_documents(self) -> None:
        raise Warning('Readonly Chroma does not support this operation. Use indexClient.addItemToIndex method instead')

    @override
    def add_texts(self) -> None:
        raise Warning('Readonly Chroma does not support this operation. Use indexClient.addItemToIndex method instead')

    @override
    def add_text(self) -> None:
        raise Warning('Readonly Chroma does not support this operation. Use indexClient.addItemToIndex method instead')

    @override
    def from_documents(self) -> None:
        raise Warning('Readonly Chroma does not support this operation. Use indexClient.addItemToIndex method instead')

    @override
    def from_texts(self) -> None:
        raise Warning('Readonly Chroma does not support this operation. Use indexClient.addItemToIndex method instead')

    @override
    def from_text(self) -> None:
        raise Warning('Readonly Chroma does not support this operation. Use indexClient.addItemToIndex method instead')

    @override
    def afrom_documents(self) -> None:
        raise Warning('Readonly Chroma does not support this operation. Use indexClient.addItemToIndex method instead')

    @override
    def afrom_texts(self) -> None:
        raise Warning('Readonly Chroma does not support this operation. Use indexClient.addItemToIndex method instead')

    @override
    def adelete(self) -> None:
        raise Warning('Readonly Chroma does not support this operation. use indexClient instead')

    @override
    def delete(self) -> None:
        raise Warning('Readonly Chroma does not support this operation. use indexClient instead')

    @override
    def delete_collection(self) -> None:
        raise Warning('Readonly Chroma does not support this operation. use indexClient instead')

    @override
    def update_documents(self) -> None:
        raise Warning('Readonly Chroma does not support this operation. use indexClient instead')

    @override
    def update_document(self) -> None:
        raise Warning('Readonly Chroma does not support this operation. use indexClient instead')



import os
from setuptools import setup, find_packages

setup(
  name='indexnetwork-sdk',
  version='0.0.13',
  description='Index Network SDK',
  long_description=open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'README.MD')).read(),
  long_description_content_type='text/markdown',
  author='Index',
  author_email='accounts@index.network',
  url='https://github.com/indexnetwork/index',
  packages=find_packages(),
  include_package_data=True,
  package_data={
      "indexnetwork_sdk": ["bin/session.bin"],
  },
  install_requires=[
    'abnf',
    'aiohttp',
    'aiosignal',
    'annotated-types',
    'asn1crypto',
    'async-timeout',
    'attrs',
    'bip-utils',
    'bitarray',
    'cbor2',
    'certifi',
    'cffi',
    'charset-normalizer',
    'ckzg',
    'coincurve',
    'crcmod',
    'crypto',
    'cytoolz',
    'ecdsa',
    'ed25519-blake2b',
    'eth-account',
    'eth-hash',
    'eth-keyfile',
    'eth-keys',
    'eth-rlp',
    'eth-typing',
    'eth-utils',
    'eth_abi',
    'frozenlist',
    'hexbytes',
    'idna',
    'jsonschema',
    'jsonschema-specifications',
    'lru-dict',
    'multidict',
    'Naked',
    'parsimonious',
    'protobuf',
    'py-sr25519-bindings',
    'py_crypto_hd_wallet',
    'pycparser',
    'pycryptodome',
    'pydantic',
    'pydantic_core',
    'PyNaCl',
    'python-dateutil',
    'pyunormalize',
    'PyYAML',
    'referencing',
    'regex',
    'requests',
    'rlp',
    'rpds-py',
    'shellescape',
    'siwe',
    'six',
    'toolz',
    'typing_extensions',
    'urllib3',
    'web3',
    'websockets',
    'wheel',
    'yarl',
  ],
  python_requires='>=3.0',
)
